@@ -124,6 +124,32 @@ FORMAT-STRING is a string specifying the message to output, as in
                         ,@objects)
      nil))
 
+(defalias 'exwm--window-inside-absolute-pixel-edges
+  (if (< emacs-major-version 31)
+      (lambda (&optional window)
+        (let* ((window (window-normalize-window window t))
+               (edges (window-inside-absolute-pixel-edges window))
+               (tab-line-height (window-tab-line-height window)))
+          (cl-incf (elt edges 1) tab-line-height)
+          (cl-incf (elt edges 3) tab-line-height)
+          edges))
+    #'window-inside-absolute-pixel-edges)
+  "A fixed implementation of `window-inside-absolute-pixel-edges'.
+This version correctly handles tab-lines on Emacs prior to v31.1.")
+
+(defalias 'exwm--window-inside-pixel-edges
+  (if (< emacs-major-version 31)
+      (lambda (&optional window)
+        (let* ((window (window-normalize-window window t))
+               (edges (window-inside-pixel-edges window))
+               (tab-line-height (window-tab-line-height window)))
+          (cl-incf (elt edges 1) tab-line-height)
+          (cl-incf (elt edges 3) tab-line-height)
+          edges))
+    #'window-inside-pixel-edges)
+  "A fixed implementation of `window-inside-pixel-edges'.
+This version correctly handles tab-lines on Emacs prior to v31.1.")
+
 (defsubst exwm--id->buffer (id)
   "X window ID => Emacs buffer."
   (declare (indent defun))
@@ -247,8 +273,8 @@ If CONN is non-nil, use it instead of the value of the variable
 (defvar-local exwm--configurations nil)   ;initial configurations.
 (defvar-local exwm--frame nil)            ;workspace frame
 (defvar-local exwm--floating-frame nil)   ;floating frame
-(defvar-local exwm--mode-line-format nil) ;save mode-line-format
-(defvar-local exwm--floating-frame-position nil) ;set when hidden.
+(defvar-local exwm--saved-mode-line-format nil) ;save mode-line-format
+(defvar-local exwm--floating-frame-geometry nil) ;set when hidden.
 (defvar-local exwm--fixed-size nil)              ;fixed size
 (defvar-local exwm--selected-input-mode 'line-mode
   "Input mode as selected by the user.
@@ -267,6 +293,7 @@ One of `line-mode' or `char-mode'.")
 (defvar-local exwm--protocols nil)
 (defvar-local exwm-state xcb:icccm:WM_STATE:NormalState "WM_STATE.")
 (defvar-local exwm--ewmh-state nil "_NET_WM_STATE.")
+(defvar-local exwm--icon nil "Application Icon (XPM string).")
 ;; _NET_WM_NORMAL_HINTS
 (defvar-local exwm--normal-hints-x nil)
 (defvar-local exwm--normal-hints-y nil)
