@@ -164,6 +164,7 @@ want to match against EXWM internal variables such as `exwm-title',
 (declare-function exwm--update-protocols "exwm.el" (id &optional force))
 (declare-function exwm--update-struts "exwm.el" (id))
 (declare-function exwm--update-title "exwm.el" (id))
+(declare-function exwm--update-icon "exwm.el" (id &optional force))
 (declare-function exwm--update-transient-for "exwm.el" (id &optional force))
 (declare-function exwm--update-desktop "exwm.el" (id &optional force))
 (declare-function exwm--update-window-type "exwm.el" (id &optional force))
@@ -300,6 +301,7 @@ This only works when procfs is mounted, which may not be the case on some BSDs."
       (exwm-manage--update-geometry id)
       (exwm-manage--update-mwm-hints id)
       (exwm--update-title id)
+      (exwm--update-icon id)
       (exwm--update-protocols id)
       (setq exwm--configurations (exwm-manage--get-configurations))
       ;; OverrideRedirect is not checked here.
@@ -341,6 +343,11 @@ This only works when procfs is mounted, which may not be the case on some BSDs."
                                ;; UnmapNotify/DestroyNotify event of the dock.
                                (exwm--get-client-event-mask)
                              xcb:EventMask:NoEvent)))
+        ;; Configure the tiling mode-line & header-line
+        (pcase-dolist (`(,var . ,setting) '((mode-line-format . tiling-mode-line)
+                                            (header-line-format . tiling-header-line)))
+          (when (plist-member exwm--configurations setting)
+            (set var (plist-get exwm--configurations setting))))
         ;; The window needs to be mapped
         (xcb:+request exwm--connection
             (make-instance 'xcb:MapWindow :window id))
@@ -664,7 +671,7 @@ border-width: %d; sibling: #x%x; stack-mode: %d"
                      (or (not exwm--floating-frame)
                          (progn
                            (setq edges
-                                 (window-inside-pixel-edges
+                                 (exwm--window-inside-pixel-edges
                                   (get-buffer-window buffer t))
                                  width-delta (- width (- (elt edges 2)
                                                          (elt edges 0)))
@@ -688,7 +695,7 @@ border-width: %d; sibling: #x%x; stack-mode: %d"
                       (with-slots (x y width height)
                           (exwm-workspace--get-geometry exwm--frame)
                         (list x y width height))
-                    (window-inside-absolute-pixel-edges
+                    (exwm--window-inside-absolute-pixel-edges
                      (get-buffer-window buffer t))))
             (exwm--log "Reply with ConfigureNotify (edges): %s" edges)
             (xcb:+request exwm--connection
@@ -712,9 +719,9 @@ border-width: %d; sibling: #x%x; stack-mode: %d"
                                                    'exwm-outer-id)
                                   nil
                                   nil
-                                  (+ (frame-pixel-width exwm--floating-frame)
+                                  (+ (frame-outer-width exwm--floating-frame)
                                      width-delta)
-                                  (+ (frame-pixel-height exwm--floating-frame)
+                                  (+ (frame-outer-height exwm--floating-frame)
                                      height-delta)))
           (exwm--log "ConfigureWindow (preserve geometry)")
           ;; Configure the unmanaged window.
@@ -804,7 +811,11 @@ SYNTHETIC indicates whether the event is a synthetic event."
   "Initialize manage module."
   ;; Intern _MOTIF_WM_HINTS
   (exwm--log)
-  (setq exwm-manage--_MOTIF_WM_HINTS (exwm--intern-atom "_MOTIF_WM_HINTS"))
+  (setq exwm-manage--_MOTIF_WM_HINTS (exwm--intern-atom "_MOTIF_WM_HINTS")
+        exwm-manage--frame-outer-id-list nil)
+  (dolist (frame (frame-list))
+    (when (display-graphic-p frame)
+      (exwm-manage--add-frame frame)))
   (add-hook 'after-make-frame-functions #'exwm-manage--add-frame)
   (add-hook 'delete-frame-functions #'exwm-manage--remove-frame)
   (xcb:+event exwm--connection 'xcb:ConfigureRequest
