@@ -1,6 +1,6 @@
 ;;; exwm-randr.el --- RandR Module for EXWM  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 
@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -42,7 +42,7 @@
 ;; to xrandr(1) for the configuration of RandR.
 
 ;; References:
-;; + RandR (http://www.x.org/archive/X11R7.7/doc/randrproto/randrproto.txt)
+;; + RandR (https://www.x.org/archive/X11R7.7/doc/randrproto/randrproto.txt)
 
 ;;; Code:
 
@@ -57,6 +57,8 @@
   "RandR."
   :group 'exwm)
 
+(defvar exwm-randr--connection nil "The X connection.")
+
 (defcustom exwm-randr-refresh-hook nil
   "Normal hook run when the RandR module just refreshed."
   :type 'hook)
@@ -86,8 +88,18 @@ setting workspace other than 1 and 3 would always be displayed on the
 primary monitor where workspace 1 and 3 would be displayed on their
 corresponding monitors whenever the monitors are active.
 
+Changes to this variable only take immediate affect when set before
+`exwm-randr-mode' is enabled, via `setopt', or when customized (see the
+Info node `Customization'). Otherwise, the `exwm-randr-refresh' must be
+called explicitly to assign the correct workspaces to the correct monitors.
+
   \\='(1 \"HDMI-1\" 3 \"DP-1\")"
-  :type '(plist :key-type integer :value-type string))
+  :type '(plist :key-type integer :value-type string)
+  :initialize 'custom-initialize-changed
+  :set (lambda (symbol value)
+         (set-default-toplevel-value symbol value)
+         (when exwm-randr--connection
+           (exwm-randr-refresh))))
 
 (defvar exwm-randr--connection nil "The X connection.")
 
@@ -106,12 +118,6 @@ corresponding monitors whenever the monitors are active.
   :group 'exwm-randr
   (exwm--global-minor-mode-body randr))
 
-(defun exwm-randr-enable ()
-  "Enable EXWM RandR support."
-  (message "`exwm-randr-enable' is obsolete. Use `exwm-randr-mode' instead.")
-  (exwm-randr-mode 1))
-(make-obsolete 'exwm-randr-enable "Use `exwm-randr-mode' instead." "0.30")
-
 (defsubst exwm-randr--assert-connected ()
   "Assert that `exwm-randr-mode' is enabled and activated."
   (cond
@@ -218,7 +224,6 @@ In a mirroring setup some monitors overlap and should be treated as one."
                                             monitor-alias-alist))))))
     monitor-alias-alist))
 
-;;;###autoload
 (defun exwm-randr-refresh ()
   "Refresh workspaces according to the updated RandR info."
   (interactive)
@@ -307,9 +312,9 @@ Refresh when any CRTC/output changes."
     (xcb:unmarshal evt data)
     (with-slots (subCode u) evt
       (cond ((= subCode xcb:randr:Notify:CrtcChange)
-	     (setq notify (slot-value u 'cc)))
-	    ((= subCode xcb:randr:Notify:OutputChange)
-	     (setq notify (slot-value u 'oc))))
+             (setq notify (slot-value u 'cc)))
+            ((= subCode xcb:randr:Notify:OutputChange)
+             (setq notify (slot-value u 'oc))))
       (when notify
         (with-slots (timestamp) notify
           (when (> timestamp exwm-randr--last-timestamp)
@@ -387,8 +392,5 @@ Refresh when any RandR 1.5 monitor changes."
     (xcb:disconnect exwm-randr--connection)
     (setq exwm-randr--connection nil)))
 
-
-
 (provide 'exwm-randr)
-
 ;;; exwm-randr.el ends here
